struct Solution;

impl Solution {
    pub fn solve_sudoku(board: &mut Vec<Vec<char>>) {
        let mut col_mask = vec![vec![false; 9]; 9];
        let mut row_mask = vec![vec![false; 9]; 9];
        let mut cell_mask = vec![vec![false; 9]; 9];

        for i in 0..9 {
            for j in 0..9 {
                if board[i][j] == '.' {
                    continue;
                }
                let idx = board[i][j] as usize - '1' as usize;
                row_mask[i][idx] = true;
                col_mask[j][idx] = true;
                let cell_idx = Self::get_cell_idx(i, j);
                cell_mask[cell_idx][idx] = true;
            }
        }

        Self::helper(board, 0, 0, &mut col_mask, &mut row_mask, &mut cell_mask);
    }

    fn helper(
        board: &mut Vec<Vec<char>>,
        row: usize,
        col: usize,
        col_mask: &mut Vec<Vec<bool>>,
        row_mask: &mut Vec<Vec<bool>>,
        cell_mask: &mut Vec<Vec<bool>>,
    ) -> bool {
        if row == 9  {
            return true;
        }

        if board[row][col] == '.' {
            for (idx, &c) in vec!['1', '2', '3', '4', '5', '6', '7', '8', '9']
                .iter()
                .enumerate()
            {
                let cell_idx = Self::get_cell_idx(row, col);


                if row_mask[row][idx] || col_mask[col][idx] || cell_mask[cell_idx][idx]{
                    continue;
                }

                board[row][col] = c;

                row_mask[row][idx] = true;
                col_mask[col][idx] = true;
                cell_mask[cell_idx][idx] = true;
                let (next_i, next_j) = Self::next_ij(row, col);

                if Self::helper(board, next_i, next_j, col_mask, row_mask, cell_mask) {
                    return true;
                } else {
                    board[row][col] = '.';

                    row_mask[row][idx] = false;
                    col_mask[col][idx] = false;
                    cell_mask[cell_idx][idx] = false;
                }
            }
        } else {
            let (next_i, next_j) = Self::next_ij(row, col);

            if Self::helper(board, next_i, next_j, col_mask, row_mask, cell_mask) {
                return true;
            } else {
                return false;
            }
        }
        return false;
    }

    fn next_ij(i: usize, j: usize) -> (usize, usize) {
        // match (i, j) {
        //     (i, 8) => (i + 1, 0),
        //     (i, j) => (i, j + 1),
        // }

        let carry = (j + 1) / 9;
        let j = (j + 1) % 9;
        (carry + i, j)
    }
    fn get_cell_idx(i: usize, j: usize) -> usize {
        match (i / 3, j / 3) {
            (0, 0) => 0,
            (0, 1) => 1,
            (0, 2) => 2,
            (1, 0) => 3,
            (1, 1) => 4,
            (1, 2) => 5,
            (2, 0) => 6,
            (2, 1) => 7,
            (2, 2) => 8,
            (_, _) => unreachable!(),
        }
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_solve_sudoku() {
        let mut board = vec![
            vec!['5', '3', '.', '.', '7', '.', '.', '.', '.'],
            vec!['6', '.', '.', '1', '9', '5', '.', '.', '.'],
            vec!['.', '9', '8', '.', '.', '.', '.', '6', '.'],
            vec!['8', '.', '.', '.', '6', '.', '.', '.', '3'],
            vec!['4', '.', '.', '8', '.', '3', '.', '.', '1'],
            vec!['7', '.', '.', '.', '2', '.', '.', '.', '6'],
            vec!['.', '6', '.', '.', '.', '.', '2', '8', '.'],
            vec!['.', '.', '.', '4', '1', '9', '.', '.', '5'],
            vec!['.', '.', '.', '.', '8', '.', '.', '7', '9'],
        ];
        Solution::solve_sudoku(&mut board);

        assert_eq!(
            board,
            vec![
                vec!['5', '3', '4', '6', '7', '8', '9', '1', '2'],
                vec!['6', '7', '2', '1', '9', '5', '3', '4', '8'],
                vec!['1', '9', '8', '3', '4', '2', '5', '6', '7'],
                vec!['8', '5', '9', '7', '6', '1', '4', '2', '3'],
                vec!['4', '2', '6', '8', '5', '3', '7', '9', '1'],
                vec!['7', '1', '3', '9', '2', '4', '8', '5', '6'],
                vec!['9', '6', '1', '5', '3', '7', '2', '8', '4'],
                vec!['2', '8', '7', '4', '1', '9', '6', '3', '5'],
                vec!['3', '4', '5', '2', '8', '6', '1', '7', '9'],
            ]
        );
    }
}
