struct Solution;

impl Solution {
    pub fn is_valid_sudoku(board: Vec<Vec<char>>) -> bool {
        for i in 0..9 {
            if !Self::is_row_valid(&board, i) {
                return false;
            }

            if !Self::is_col_valid(&board, i) {
                return false;
            }
        }

        for i in 0..3 {
            for j in 0..3 {
                if !Self::is_cell_valid(&board, i, j) {
                    return false;
                }
            }
        }

        true
    }

    fn is_row_valid(board: &Vec<Vec<char>>, i: usize) -> bool {
        let mut m = std::collections::HashSet::new();
        for j in 0..9 {
            if board[i][j] == '.' {
                continue;
            }
            match m.get(&board[i][j]) {
                Some(_) => {
                    return false;
                }
                None => {
                    m.insert(board[i][j]);
                }
            }
        }
        return true;
    }

    fn is_col_valid(board: &Vec<Vec<char>>, j: usize) -> bool {
        let mut m = std::collections::HashSet::new();
        for i in 0..9 {
            if board[i][j] == '.' {
                continue;
            }
            match m.get(&board[i][j]) {
                Some(_) => {
                    return false;
                }
                None => {
                    m.insert(board[i][j]);
                }
            }
        }
        return true;
    }

    fn is_cell_valid(board: &Vec<Vec<char>>, row: usize, col: usize) -> bool {
        let mut m = std::collections::HashSet::new();
        for i in 0..3 {
            for j in 0..3 {
                if board[row * 3 + i][col * 3 + j] == '.' {
                    continue;
                }
                match m.get(&board[row * 3 + i][col * 3 + j]) {
                    Some(_) => {
                        return false;
                    }
                    None => {
                        m.insert(board[row * 3 + i][col * 3 + j]);
                    }
                }
            }
        }
        return true;
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_is_valid_sudoku() {
        assert_eq!(
            Solution::is_valid_sudoku(vec![
                vec!['5', '3', '.', '.', '7', '.', '.', '.', '.'],
                vec!['6', '.', '.', '1', '9', '5', '.', '.', '.'],
                vec!['.', '9', '8', '.', '.', '.', '.', '6', '.'],
                vec!['8', '.', '.', '.', '6', '.', '.', '.', '3'],
                vec!['4', '.', '.', '8', '.', '3', '.', '.', '1'],
                vec!['7', '.', '.', '.', '2', '.', '.', '.', '6'],
                vec!['.', '6', '.', '.', '.', '.', '2', '8', '.'],
                vec!['.', '.', '.', '4', '1', '9', '.', '.', '5'],
                vec!['.', '.', '.', '.', '8', '.', '.', '7', '9'],
            ]),
            true
        );
    }

    #[test]
    fn test_is_valid_sudoku_false() {
        assert_eq!(
            Solution::is_valid_sudoku(vec![
                vec!['8', '3', '.', '.', '7', '.', '.', '.', '.'],
                vec!['6', '.', '.', '1', '9', '5', '.', '.', '.'],
                vec!['.', '9', '8', '.', '.', '.', '.', '6', '.'],
                vec!['8', '.', '.', '.', '6', '.', '.', '.', '3'],
                vec!['4', '.', '.', '8', '.', '3', '.', '.', '1'],
                vec!['7', '.', '.', '.', '2', '.', '.', '.', '6'],
                vec!['.', '6', '.', '.', '.', '.', '2', '8', '.'],
                vec!['.', '.', '.', '4', '1', '9', '.', '.', '5'],
                vec!['.', '.', '.', '.', '8', '.', '.', '7', '9'],
            ]),
            false
        );
    }
}
